@@ -1,48 +1,44 @@
 """Library defining the interface to a project."""
 import json
-from typing import Any, Dict, Iterator, List, NamedTuple, Optional, cast
+from datetime import datetime, timedelta
+from http import HTTPStatus
+from typing import Any, Dict, Iterator, List, NamedTuple, Optional, Tuple, cast
 
-import grpc
-from google.protobuf.timestamp_pb2 import Timestamp
-
-from rime_sdk.firewall import Firewall, location_args_to_data_location
-from rime_sdk.internal.backend import RIMEBackend
-from rime_sdk.internal.proto_utils import get_bin_size_proto, get_threshold_info_proto
+from rime_sdk.firewall import Firewall, get_reference_set_source
+from rime_sdk.internal.config_parser import convert_config_to_swagger
+from rime_sdk.internal.rest_error_handler import RESTErrorHandler
+from rime_sdk.internal.swagger_utils import (
+    get_bin_size_swagger,
+    get_data_type_enum_swagger,
+    get_threshold_info_swagger,
+    location_args_to_data_location_swagger,
+)
 from rime_sdk.internal.utils import convert_dict_to_html, make_link
-from rime_sdk.protos.feature_flag.feature_flag_pb2 import (
-    GetLimitStatusRequest,
-    LicenseLimitName,
-    LimitStatus,
-)
-from rime_sdk.protos.firewall.firewall_pb2 import (
-    ConvertIDsRequest,
-    ConvertIDsResponse,
-    CreateFirewallFromComponentsRequest,
-    CreateFirewallFromTestRunIDRequest,
-    CreateFirewallResponse,
-    FirewallComponents,
-    FirewallConvIDType,
-    FirewallRules,
-)
-from rime_sdk.protos.notification.notification_pb2 import (
-    CreateNotificationRequest,
-    DeleteNotificationRequest,
-    DigestConfig,
-    JobActionConfig,
-    ListNotificationsRequest,
-    ListNotificationsResponse,
-    MonitoringConfig,
-    Notification,
-    NotificationType,
-    UpdateNotificationRequest,
-    WebhookConfig,
-)
-from rime_sdk.protos.project.project_pb2 import AnnotatedProject, GetProjectRequest
-from rime_sdk.protos.result_synthesizer.result_message_pb2 import CLIConfig
-from rime_sdk.protos.test_run_results.test_run_results_pb2 import (
-    ListTestRunsRequest,
-    ListTestRunsResponse,
+from rime_sdk.swagger import swagger_client
+from rime_sdk.swagger.swagger_client import ApiClient
+from rime_sdk.swagger.swagger_client.models import (
+    ComponentsProjectIdBody,
+    DigestConfigDigestFrequency,
+    FirewallProjectIdBody,
+    NotificationObjectType,
+    RimeAnnotatedProject,
+    RimeConfig,
+    RimeCreateFirewallResponse,
+    RimeCreateNotificationRequest,
+    RimeDigestConfig,
+    RimeFirewallComponents,
+    RimeFirewallRules,
+    RimeJobActionConfig,
+    RimeLicenseLimit,
+    RimeLimitStatusStatus,
+    RimeListNotificationsResponse,
+    RimeManagedImageReference,
+    RimeMonitoringConfig,
+    RimeNotificationType,
+    RimeWebhookConfig,
+    TestrunresultGetTestRunResponse,
 )
+from rime_sdk.swagger.swagger_client.rest import ApiException
 from rime_sdk.test_run import TestRun
 
 NOTIFICATION_TYPE_JOB_ACTION_STR: str = "Job_Action"
@@ -73,22 +69,22 @@ class Project:
     This object provides an interface for editing, updating, and deleting projects.
 
     Attributes:
-        backend: RIMEBackend
-            The RIME backend used to query about the status of the job.
+        api_client: ApiClient
+                The client used to query about the status of the job.
         project_id: str
             The identifier for the RIME project that this object monitors.
     """
 
-    def __init__(self, backend: RIMEBackend, project_id: str) -> None:
+    def __init__(self, api_client: ApiClient, project_id: str) -> None:
         """Contains information about a RIME Project.
 
         Args:
-            backend: RIMEBackend
-                The RIME backend used to query about the status of the job.
+            api_client: ApiClient
+                The client used to query about the status of the job.
             project_id: str
                 The identifier for the RIME project that this object monitors.
         """
-        self._backend = backend
+        self._api_client = api_client
         self._project_id = project_id
 
     def __repr__(self) -> str:
@@ -115,24 +111,27 @@ class Project:
             ValueError if another firewall cannot be created as it would
             exceed license limits.
         """
-        req = GetLimitStatusRequest(
-            customer_name=self._backend.customer_name,
-            limit_name=LicenseLimitName.LICENSE_LIMIT_NAME_FIREWALL,
-        )
-        with self._backend.get_feature_flag_stub() as feature_flag_client:
-            with self._backend.GRPCErrorHandler():
-                feature_flag_response = feature_flag_client.GetLimitStatus(req)
+        api = swagger_client.RIMEInfoApi(self._api_client)
+        with RESTErrorHandler():
+            rime_info_response = api.r_ime_info_get_rime_info()
+
+        feature_flag_api = swagger_client.FeatureFlagApi(self._api_client)
+        with RESTErrorHandler():
+            feature_flag_response = feature_flag_api.feature_flag_get_limit_status(
+                customer_name=rime_info_response.customer_name,
+                limit=RimeLicenseLimit.FIREWALL,
+            )
 
         limit_status = feature_flag_response.limit_status.limit_status
         limit_value = feature_flag_response.limit_status.limit_value
-        if limit_status == LimitStatus.Status.WARN:
-            curr_value = feature_flag_response.limit_status.current_value
+        if limit_status == RimeLimitStatusStatus.WARN:
+            curr_value = int(feature_flag_response.limit_status.current_value)
             print(
                 f"You are approaching the limit ({curr_value + 1}"
                 f"/{limit_value}) of models monitored. Contact the"
                 f" Robust Intelligence team to upgrade your license."
             )
-        elif limit_status == LimitStatus.Status.ERROR:
+        elif limit_status == RimeLimitStatusStatus.ERROR:
             # could be either within grace period or exceeded grace period
             # if the latter, let the create firewall call raise the
             # error
@@ -141,22 +140,33 @@ class Project:
                 " Contact the Robust Intelligence team to"
                 " upgrade your license."
             )
-        elif limit_status == LimitStatus.Status.OK:
+        elif limit_status == RimeLimitStatusStatus.OK:
             pass
         else:
             raise ValueError("Unexpected status value.")
 
-    def _get_project(self) -> AnnotatedProject:
+    def _get_data_type(self, test_run_id: str) -> str:
+        """Get the Data Type associated with this test run."""
+        with RESTErrorHandler():
+            api = swagger_client.ResultsReaderApi(self._api_client)
+            res: TestrunresultGetTestRunResponse = api.results_reader_get_test_run(
+                test_run_id=test_run_id,
+            )
+        if res.test_run:
+            return res.test_run.data_type
+        else:
+            raise ValueError("no test run found")
+
+    def _get_project(self) -> RimeAnnotatedProject:
         """Get the project info from the backend.
 
         Returns:
             A ``GetProjectResponse`` object.
         """
-        req = GetProjectRequest(project_id=self._project_id)
-        with self._backend.get_project_manager_stub() as project_client:
-            with self._backend.GRPCErrorHandler():
-                response = project_client.GetProject(req)
-        return response.project
+        api = swagger_client.ProjectManagerApi(self._api_client)
+        with RESTErrorHandler():
+            response = api.project_manager_get_project(self._project_id)
+            return response.project
 
     @property
     def info(self) -> ProjectInfo:
@@ -191,22 +201,22 @@ class Project:
 
     def list_test_runs(self) -> Iterator[TestRun]:
         """List the stress test runs associated with the project."""
-        with self._backend.get_test_run_results_stub() as test_run_results:
-            # Iterate through the pages of projects and break at the last page.
-            page_token = ""
-            while True:
-                if page_token == "":
-                    request = ListTestRunsRequest(project_id=self._project_id,)
-                else:
-                    request = ListTestRunsRequest(page_token=page_token)
-                res: ListTestRunsResponse = test_run_results.ListTestRuns(request)
+        api = swagger_client.ResultsReaderApi(self._api_client)
+        # Iterate through the pages of projects and break at the last page.
+        page_token = ""
+        while True:
+            if page_token == "":
+                res = api.results_reader_list_test_runs(project_id=self._project_id)
+            else:
+                res = api.results_reader_list_test_runs(page_token=page_token)
+            if res.test_runs is not None:
                 for test_run in res.test_runs:
-                    yield TestRun(self._backend, test_run.test_run_id)
-                # Advance to the next page of test cases.
-                page_token = res.next_page_token
-                # we've reached the last page of test cases.
-                if not res.has_more:
-                    break
+                    yield TestRun(self._api_client, test_run.test_run_id)
+            # Advance to the next page of test cases.
+            page_token = res.next_page_token
+            # we've reached the last page of test cases.
+            if not res.has_more:
+                break
 
     def create_firewall(
         self,
@@ -214,11 +224,14 @@ class Project:
         bin_size: str,
         test_run_id: str,
         run_ct_schedule: bool = False,
+        rolling_window_duration: Optional[timedelta] = None,
+        reference_set_time_bin: Optional[Tuple[datetime, datetime]] = None,
         location_type: Optional[str] = None,
+        location_info: Optional[Dict] = None,
+        data_params: Optional[Dict] = None,
+        rime_managed_image: Optional[str] = None,
     ) -> Firewall:  # noqa: D400, D402
-        """create_firewall(name: str, bin_size: str, test_run_id: str) -> Firewall
-
-        Create a Firewall for a given project.
+        """Create a Firewall for a given project.
 
         Args:
             name: str
@@ -227,6 +240,29 @@ class Project:
                 Bin size. Can be `year`, `month`, `week`, `day`, `hour`.
             test_run_id: str
                 ID of the stress test run that firewall will be based on.
+            run_ct_schedule: bool
+                Flag for ct scheduler.
+            rolling_window_duration: Optional[int]
+                Time duration of rolling window of reference set if provided.
+                The rolling window is only supported for firewall running scheduled ct.
+                Only one of rolling_window_seconds or reference_set_time_bin may be set.
+            reference_set_time_bin: Optional[Tuple[datetime, datetime]]
+                Time bin of reference set can be set for firewall running scheduled ct.
+                Only one of rolling_window_seconds or reference_set_time_bin may be set.
+            location_type: Optional[str]
+                Type of the data location that ScheduledCT will pull data from.
+            location_info: Optional[Dict]
+                Information needed to access the data location provided.
+            data_params: Optional[Dict]
+                Information needed to process data from the data location provided.
+                By default, these are obtained from your reference information.
+                Eg. `pred_col`, `timestamp_col`, `label_col`, etc...
+            rime_managed_image: Optional[str]
+                Name of a managed image to use when running the model test.
+                The image must have all dependencies required by your model. To create
+                new managed images with your desired dependencies, use the client's
+                ``create_managed_image()`` method.
+
 
         Returns:
             A ``Firewall`` object.
@@ -245,29 +281,44 @@ class Project:
                 "firewall name", "day", "foo")
         """
         self._check_firewall_creation_limit()
-        bin_size_proto = get_bin_size_proto(bin_size_str=bin_size)
-        req = CreateFirewallFromTestRunIDRequest(
+        bin_size_swagger = get_bin_size_swagger(bin_size_str=bin_size)
+        reference_set_source = get_reference_set_source(
+            rolling_window_duration=rolling_window_duration,
+            reference_set_time_bin=reference_set_time_bin,
+        )
+        req = FirewallProjectIdBody(
             name=name,
-            project_id=self._project_id,
-            bin_size=bin_size_proto,
+            bin_size=bin_size_swagger,
             run_ct_schedule=run_ct_schedule,
             stress_test_run_id=test_run_id,
+            reference_set_source=reference_set_source,
         )
 
+        if rime_managed_image:
+            req.managed_image = RimeManagedImageReference(name=rime_managed_image)
+
+        # Prevent location info from being provided without location type
+        if location_info is not None and location_type is None:
+            raise ValueError("Must Specify both location type and location info.")
         if location_type is not None:
-            location_info = location_args_to_data_location(location_type)
-            req.data_location_info.CopyFrom(location_info)
+            swagger_data_type = self._get_data_type(test_run_id)
+            location_args = location_args_to_data_location_swagger(
+                location_type, location_info, data_params, swagger_data_type
+            )
+            req.data_location_info = location_args
         try:
-            with self._backend.get_firewall_stub() as firewall_tester:
-                res = firewall_tester.CreateFirewallFromTestRunID(req)
-                res = cast(CreateFirewallResponse, res)
-                return Firewall(self._backend, res.firewall_id)
-        except grpc.RpcError as rpc_error:
-            if rpc_error.code() == grpc.StatusCode.NOT_FOUND:
+            api = swagger_client.FirewallServiceApi(self._api_client)
+            res = api.firewall_service_create_firewall_from_test_run_id(
+                body=req, project_id=self._project_id
+            )
+            res = cast(RimeCreateFirewallResponse, res)
+            return Firewall(self._api_client, res.firewall_id)
+        except ApiException as e:
+            if e.status == HTTPStatus.NOT_FOUND:
                 raise ValueError(
                     f"a test run with this id (`{test_run_id}`)  does not exist"
                 )
-            raise ValueError(rpc_error.details()) from None
+            raise ValueError(e.reason) from None
 
     def create_firewall_from_components(
         self,
@@ -278,19 +329,43 @@ class Project:
         threshold_infos: List[dict],
         run_ct_schedule: bool = False,
         location_type: Optional[str] = None,
+        location_info: Optional[Dict] = None,
+        data_params: Optional[Dict] = None,
+        rime_managed_image: Optional[str] = None,
+        data_type: str = "tabular",
     ) -> Firewall:  # noqa: D400, D402
-        """create_firewall_from_components(name: str, bin_size: str, stress_test_config: Dict[str, Any], firewall_rules: List[Dict[str, Any]], threshold_infos: List[dict]) -> Firewall
-
-        Create a Firewall for a given project.
+        """Create a Firewall for a given project.
 
         Args:
             name: str
                 FW name.
-            bin_size: str Can be `year`, `month`, `week`, `day`, `hour`.
-            stress_test_config: RIME Config that indicates the testing, model, and
-                data configurations
-            firewall_rules: Firewall Rules to update the firewall with.
-            threshold_infos: Threshold info for each summary metric.
+            bin_size: str
+                Can be `year`, `month`, `week`, `day`, `hour`.
+            stress_test_config: dict
+                RIME Config that indicates the testing, model, and data configurations
+            firewall_rules: List[Dict]
+                Firewall Rules to update the firewall with.
+            threshold_infos: List[Dict]
+                Threshold info for each summary metric.
+            run_ct_schedule: bool
+                Flag for ct scheduler.
+            location_type: Optional[str]
+                Type of the data location that ScheduledCT will pull data from.
+            location_info:  Optional[Dict]
+                Information needed to access the data location provided.
+            data_params: Optional[Dict]
+                Information needed to process data from the data location provided.
+                By default, these are obtained from your reference information.
+                Eg. `pred_col`, `timestamp_col`, `label_col`, etc...
+            rime_managed_image: Optional[str]
+                Name of a managed image to use when running the model test.
+                The image must have all dependencies required by your model. To create
+                new managed images with your desired dependencies, use the client's
+                ``create_managed_image()`` method.
+            data_type: str
+                Type of data this firewall test is to be run on. Should be one of
+                `tabular`, `nlp`, `images`. Defaults to `tabular`.
+
 
         Returns:
             A ``Firewall`` object.
@@ -346,53 +421,53 @@ class Project:
                 metric_thresholds,
             )
         """
-        bin_size_proto = get_bin_size_proto(bin_size_str=bin_size)
-        cli_config_pb = CLIConfig(data=json.dumps(stress_test_config).encode("utf-8"))
-        firewall_rules_pb = FirewallRules(
+        bin_size_swagger = get_bin_size_swagger(bin_size_str=bin_size)
+        swagger_data_type = get_data_type_enum_swagger(data_type)
+        typed_cli_config = convert_config_to_swagger(
+            stress_test_config, swagger_data_type
+        )
+        firewall_rules_swagger = RimeFirewallRules(
             data=json.dumps(firewall_rules).encode("utf-8")
         )
         metric_thresholds = [
-            get_threshold_info_proto(threshold_dict)
+            get_threshold_info_swagger(threshold_dict)
             for threshold_dict in threshold_infos
         ]
-        req = CreateFirewallFromComponentsRequest(
+        req = ComponentsProjectIdBody(
             name=name,
-            project_id=self._project_id,
-            bin_size=bin_size_proto,
+            bin_size=bin_size_swagger,
             run_ct_schedule=run_ct_schedule,
-            components=FirewallComponents(
-                cli_config=cli_config_pb,
-                firewall_rules=firewall_rules_pb,
+            components=RimeFirewallComponents(
+                typed_cli_config=typed_cli_config,
+                firewall_rules=firewall_rules_swagger,
                 threshold_infos=metric_thresholds,
             ),
         )
+
+        if rime_managed_image:
+            req.managed_image = RimeManagedImageReference(name=rime_managed_image)
+
+        # Prevent location info from being provided without location type
+        if location_info is not None and location_type is None:
+            raise ValueError("Must Specify both location type and location info.")
         if location_type is not None:
-            location_info = location_args_to_data_location(location_type)
-            req.data_location_info.CopyFrom(location_info)
-        with self._backend.GRPCErrorHandler():
-            with self._backend.get_firewall_stub() as firewall_tester:
-                res = firewall_tester.CreateFirewallFromComponents(req)
-                res = cast(CreateFirewallResponse, res)
-                return Firewall(self._backend, res.firewall_id)
+            location_args = location_args_to_data_location_swagger(
+                location_type, location_info, data_params, swagger_data_type
+            )
+            req.data_location_info = location_args
+        with RESTErrorHandler():
+            api = swagger_client.FirewallServiceApi(self._api_client)
+            res = api.firewall_service_create_firewall_from_components(
+                body=req, project_id=self._project_id
+            )
+            res = cast(RimeCreateFirewallResponse, res)
+            return Firewall(self._api_client, res.firewall_id)
 
     def _get_firewall_id(self) -> Optional[str]:
-        src_type = FirewallConvIDType.FIREWALL_CONV_ID_TYPE_PROJECT_ID
-        dst_type = FirewallConvIDType.FIREWALL_CONV_ID_TYPE_FIREWALL_ID
-        req = ConvertIDsRequest(
-            src_type=src_type, dst_type=dst_type, src_ids=[self._project_id]
-        )
-        with self._backend.GRPCErrorHandler():
-            with self._backend.get_firewall_stub() as firewall_tester:
-                res: ConvertIDsResponse = firewall_tester.ConvertIDs(req)
-        src_dst_id_mapping = res.src_dst_id_mapping
-        if self._project_id not in src_dst_id_mapping:
-            return None
-        # Current backend functionality is to return mapping for everything,
-        # but with empty string if no firewall exists.
-        firewall_id = src_dst_id_mapping.get(self._project_id, "")
-        if firewall_id == "":
-            return None
-        return firewall_id
+        api = swagger_client.ProjectManagerApi(self._api_client)
+        with RESTErrorHandler():
+            response = api.project_manager_get_project(project_id=self._project_id)
+            return response.project.project.firewall_id
 
     def get_firewall(self) -> Firewall:
         """Get the active Firewall for a project if it exists.
@@ -418,7 +493,7 @@ class Project:
         firewall_id = self._get_firewall_id()
         if firewall_id is None:
             raise ValueError("No firewall found for given project.")
-        return Firewall(self._backend, firewall_id)
+        return Firewall(self._api_client, firewall_id)
 
     def has_firewall(self) -> bool:
         """Check whether a project has a firewall or not."""
@@ -430,49 +505,45 @@ class Project:
         firewall = self.get_firewall()
         firewall.delete_firewall()
 
-    def _list_notification_settings(self) -> ListNotificationsResponse:
+    def _list_notification_settings(self) -> RimeListNotificationsResponse:
         """Get list of notifications associated with the current project."""
-        req = ListNotificationsRequest(
-            list_notifications_query=ListNotificationsRequest.ListNotificationsQuery(
-                notification_object_ids=[self.project_id],
+        api = swagger_client.NotificationSettingApi(self._api_client)
+        with RESTErrorHandler():
+            response = api.notification_setting_list_notifications(
+                list_notifications_query_notification_object_ids=[self._project_id]
             )
-        )
-        with self._backend.get_notification_settings_stub() as notif_stub:
-            res = notif_stub.ListNotifications(req)
-            return res
+            return response
 
     def _set_create_notification_setting_config_from_type(
-        self, req: CreateNotificationRequest, notif_type: int
+        self, req: RimeCreateNotificationRequest, notif_type: str
     ) -> None:
-        if notif_type == NotificationType.NOTIFICATION_TYPE_JOB_ACTION:
-            req.config.job_action.CopyFrom(JobActionConfig())
-        elif notif_type == NotificationType.NOTIFICATION_TYPE_MONITORING:
-            req.config.monitoring_config.CopyFrom(MonitoringConfig())
-        elif notif_type == NotificationType.NOTIFICATION_TYPE_DIGEST:
-            timestamp = Timestamp()
-            timestamp.GetCurrentTime()
-            req.config.digest_config.CopyFrom(
-                DigestConfig(frequency=DigestConfig.DAILY, start_time=timestamp,)
+        if notif_type == RimeNotificationType.JOB_ACTION:
+            req.config.job_action = RimeJobActionConfig()
+        elif notif_type == RimeNotificationType.MONITORING:
+            req.config.monitoring_config = RimeMonitoringConfig()
+        elif notif_type == RimeNotificationType.DIGEST:
+            req.config.digest_config = RimeDigestConfig(
+                frequency=DigestConfigDigestFrequency.DAILY
             )
 
-    def _get_notification_type_from_str(self, notif_type: str) -> int:
+    def _get_notification_type_from_str(self, notif_type: str) -> str:
         if notif_type == NOTIFICATION_TYPE_JOB_ACTION_STR:
-            return NotificationType.NOTIFICATION_TYPE_JOB_ACTION
+            return RimeNotificationType.JOB_ACTION
         elif notif_type == NOTIFICATION_TYPE_MONITORING_STR:
-            return NotificationType.NOTIFICATION_TYPE_MONITORING
+            return RimeNotificationType.MONITORING
         elif notif_type == NOTIFICATION_TYPE_DIGEST_STR:
-            return NotificationType.NOTIFICATION_TYPE_DIGEST
+            return RimeNotificationType.DIGEST
         else:
             raise ValueError(
                 f"Notification type must be one of {NOTIFICATION_TYPES_STR_LIST}"
             )
 
-    def _get_notification_type_str(self, notif_type: int) -> str:
-        if notif_type == NotificationType.NOTIFICATION_TYPE_JOB_ACTION:
+    def _get_notification_type_str(self, notif_type: str) -> str:
+        if notif_type == RimeNotificationType.JOB_ACTION:
             return NOTIFICATION_TYPE_JOB_ACTION_STR
-        elif notif_type == NotificationType.NOTIFICATION_TYPE_MONITORING:
+        elif notif_type == RimeNotificationType.MONITORING:
             return NOTIFICATION_TYPE_MONITORING_STR
-        elif notif_type == NotificationType.NOTIFICATION_TYPE_DIGEST:
+        elif notif_type == RimeNotificationType.DIGEST:
             return NOTIFICATION_TYPE_DIGEST_STR
         else:
             # This function is called only to show the user notification types
@@ -516,7 +587,7 @@ class Project:
         self,
         notif_type_str: str,
         email: Optional[str],
-        webhook_config: Optional[WebhookConfig],
+        webhook_config: Optional[RimeWebhookConfig],
     ) -> None:
         """Add the email or webhook in the notification settings of notif_type.
 
@@ -526,6 +597,7 @@ class Project:
         a notification object exists for the give notification type and appends
         the email/webhook if found, else it creates a new notification object
         """
+        api = swagger_client.NotificationSettingApi(self._api_client)
         if email is not None and webhook_config is not None:
             raise ValueError(
                 "_add_notif_entry expects exactly one of email or "
@@ -539,10 +611,8 @@ class Project:
                     for existing_email in notif_setting.emails:
                         if existing_email == email:
                             print(
-                                "Email: %s already exists in notification "
-                                "settings for notification type: %s",
-                                email,
-                                notif_type_str,
+                                f"Email: {email} already exists in notification "
+                                f"settings for notification type: {notif_type_str}"
                             )
                             return
                     notif_setting.emails.append(email)
@@ -550,21 +620,24 @@ class Project:
                     for existing_webhook in notif_setting.webhooks:
                         if existing_webhook.webhook == webhook_config.webhook:
                             print(
-                                "Webhook: %s already exists in notification "
-                                "settings for notification type: %s",
-                                webhook_config.webhook,
-                                notif_type_str,
+                                f"Webhook: {webhook_config.webhook} "
+                                "already exists in notification settings "
+                                f"for notification type: {notif_type_str}"
                             )
                             return
                     notif_setting.webhooks.append(webhook_config)
-                update_req = UpdateNotificationRequest(notification=notif_setting)
-                with self._backend.get_notification_settings_stub() as notif_stub:
-                    notif_stub.UpdateNotification(update_req)
-                return
+                with RESTErrorHandler():
+                    api.notification_setting_update_notification(
+                        body=notif_setting, notification_id=notif_setting.id
+                    )
+                    return
         # Notification setting does not exist for the notif_type.
-        req = CreateNotificationRequest(
-            notification_object_type=Notification.PROJECT,
+        req = RimeCreateNotificationRequest(
+            notification_object_type=NotificationObjectType.PROJECT,
             notification_object_id=self.project_id,
+            config=RimeConfig(),
+            emails=[],
+            webhooks=[],
         )
         self._set_create_notification_setting_config_from_type(req, notif_type)
         notif_entry_str = ""
@@ -574,15 +647,16 @@ class Project:
         elif webhook_config is not None:
             req.webhooks.append(webhook_config)
             notif_entry_str = "Webhook " + webhook_config.webhook
-        with self._backend.get_notification_settings_stub() as notif_stub:
-            notif_stub.CreateNotification(req)
-        print("%s added for notification type %s", notif_entry_str, notif_type_str)
+        with RESTErrorHandler():
+            api.notification_setting_create_notification(body=req)
+            print(f"{notif_entry_str} added for notification type {notif_type_str}")
+            return
 
     def _remove_notif_entry(
         self,
         notif_type_str: str,
         email: Optional[str],
-        webhook_config: Optional[WebhookConfig],
+        webhook_config: Optional[RimeWebhookConfig],
     ) -> None:
         """Remove the email or webhook in the notification settings of notif_type.
 
@@ -613,37 +687,39 @@ class Project:
                             notif_setting.webhooks.remove(existing_webhook)
                             found = True
                 if found:
-                    if (
-                        len(notif_setting.emails) == 0
-                        and len(notif_setting.webhooks) == 0
-                    ):
-                        del_req = DeleteNotificationRequest(id=notif_setting.id)
-                        with self._backend.get_notification_settings_stub() as notif_s:
-                            notif_s.DeleteNotification(del_req)
-                    else:
-                        update_req = UpdateNotificationRequest(
-                            notification=notif_setting
-                        )
-                        with self._backend.get_notification_settings_stub() as notif_s:
-                            notif_s.UpdateNotification(update_req)
-                    return
+                    api = swagger_client.NotificationSettingApi(self._api_client)
+                    with RESTErrorHandler():
+                        if (
+                            len(notif_setting.emails) == 0
+                            and len(notif_setting.webhooks) == 0
+                        ):
+                            api.notification_setting_delete_notification(
+                                id=notif_setting.id
+                            )
+                        else:
+                            body = {"notification": notif_setting}
+                            api.notification_setting_update_notification(
+                                body=body, notification_id=notif_setting.id
+                            )
+                        return
         notif_entry_str = ""
         if email is not None:
             notif_entry_str = "Email " + email
         elif webhook_config is not None:
             notif_entry_str = "Webhook " + webhook_config.webhook
-        print("%s not found for notification type %s", notif_entry_str, notif_type_str)
+        print(f"{notif_entry_str} not found for notification type {notif_type_str}")
 
     def add_email(self, email: str, notif_type_str: str) -> None:
+        # pylint: disable=line-too-long
         """Add an email to the notification settings for the given notification type.
 
         Currently, we support 3 notification types:
         ["Job_Action", "Monitoring", "Daily_Digest"]
 
         Example:
-        .. code-block:: python
+            .. code-block:: python
 
-            notification_settings = project.add_email("<email>", "<notification type>")
+                notification_settings = project.add_email("<email>", "<notification type>")
         """
         if email == "":
             raise ValueError("Email must be a non empty string")
@@ -652,16 +728,16 @@ class Project:
         )
 
     def remove_email(self, email: str, notif_type_str: str) -> None:
+        # pylint: disable=line-too-long
         """Remove an email from notification settings for the given notification type.
 
         Currently, we support 3 notification types:
         ["Job_Action", "Monitoring", "Daily_Digest"]
 
         Example:
-        .. code-block:: python
+            .. code-block:: python
 
-            notification_settings = project.remove_email("<email>",
-            "<notification type>")
+                notification_settings = project.remove_email("<email>", "<notification type>")
         """
         if email == "":
             raise ValueError("Email must be a non empty string")
@@ -670,39 +746,51 @@ class Project:
         )
 
     def add_webhook(self, webhook: str, notif_type_str: str) -> None:
+        # pylint: disable=line-too-long
         """Add a webhook to the notification settings for the given notification type.
 
         Currently, we support 3 notification types:
         ["Job_Action", "Monitoring", "Daily_Digest"]
 
         Example:
-        .. code-block:: python
+            .. code-block:: python
 
-            notification_settings = project.add_webhook("<webhook>",
-            "<notification type>")
+                notification_settings = project.add_webhook("<webhook>", "<notification type>")
         """
         if webhook == "":
             raise ValueError("Webhook must be a non empty string")
-        webhook_config = WebhookConfig(webhook=webhook)
+        webhook_config = RimeWebhookConfig(webhook=webhook)
         return self._add_notif_entry(
             notif_type_str=notif_type_str, email=None, webhook_config=webhook_config
         )
 
     def remove_webhook(self, webhook: str, notif_type_str: str) -> None:
+        # pylint: disable=line-too-long,
         """Remove a webhook from notification settings for the given notification type.
 
         Currently, we support 3 notification types:
         ["Job_Action", "Monitoring", "Daily_Digest"]
 
         Example:
-        .. code-block:: python
+            .. code-block:: python
 
-            notification_settings = project.remove_webhook("<webhook>",
-            "<notification type>")
+                notification_settings = project.remove_webhook("<webhook>", "<notification type>")
         """
         if webhook == "":
             raise ValueError("Webhook must be a non empty string")
-        webhook_config = WebhookConfig(webhook=webhook)
+        webhook_config = RimeWebhookConfig(webhook=webhook)
         return self._remove_notif_entry(
             notif_type_str=notif_type_str, email=None, webhook_config=webhook_config
         )
+
+    def delete(self) -> None:
+        """Delete project in RIME's backend."""
+        api = swagger_client.ProjectManagerApi(self._api_client)
+        try:
+            api.project_manager_delete_project(self._project_id)
+        except ApiException as e:
+            if e.status == HTTPStatus.NOT_FOUND:
+                raise ValueError(
+                    f"project with this id {self._project_id} does not exist"
+                )
+            raise ValueError(e.reason)
